#pragma once
#include <string>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <vector>
#include <time.h>
#include <iterator>
#include <list>

class MyException : public std::exception
{
	const char* exception;
public:
	MyException(const char* exception_p)
	{
		this->exception = exception_p;
	}
	const char* what() const { return this->exception; }
};

void createData(const std::string& filename)
{
	srand(time(nullptr));
	std::ofstream file(filename, std::ios::trunc);
	for (int i = 0; i < 100; i++)
	{
		file << rand() % 101 - 50 << " ";
	}
	file.close();
}

void createDataIterator(const std::string& filename)
{
	srand(time(nullptr));
	std::vector<int> vec(100);
	std::generate(vec.begin(), vec.end(), []() { return rand() % 101 - 50; });
	std::ofstream file(filename, std::ios::trunc);
	std::ostream_iterator<int> os(file, " ");
	copy(vec.begin(), vec.end(), os);
	file.close(); //!!! Файл и так закрывается деструктором
}

template<typename T>
T loadData(const std::string& filename)
{
	std::ifstream file(filename);
	if (file.fail())
		throw MyException("Could not find file");
	T container;
	typename T::value_type tmp;
	while (file >> tmp)
	{
		container.push_back(tmp);
	}
	file.close();
	return container;
}

template<typename T>
T loadDataIterator(const std::string& filename)
{
	T container;
	std::ifstream file(filename);
	if (file.fail())
		throw MyException("Could not find file");
	std::istream_iterator<typename T::value_type> is(file);
	std::istream_iterator<typename T::value_type> end_of_file;
	std::copy(is, end_of_file, std::inserter(container, container.end()));
	file.close(); //!!! Файл и так закрывается деструктором
	return container;
}

template < typename T >
void modify(T& cont)
{
	if (cont.size() == 0)
		throw MyException("Can not modify empty container");
		
		
	double i; //!!! Использование явного типа
	
	for (auto j : cont) //!!! Копирование на каждой итерации
	{
		if (j < 0)
		{
			i = j;
			break;
		}
	}
	auto it = cont.begin();
	for (int i = 0; i < 50; i++)
	{
		it++;
	}
	i = (i + *it) / 2;

	typename T::value_type sum = 0; //!!! Не 0, а = T::value_type()
	typename T::value_type sum_abs = 0; //!!! Не 0, а = T::value_type()
	for (auto it = cont.begin(); it != cont.end(); ++it)
	{
		*it /= i;
		sum += *it;
		sum_abs += abs(*it);
	}
	cont.push_back(sum);
	cont.push_back(sum / cont.size());
}

template<typename T>
void modifyIter(T beg, T end)
{
	if (beg == end)
		throw MyException("Can not modify empty container");
	double i; //!!! Использование явного типа
	for (auto j = beg; j != end; j++)
	{
		if (*j < 0)
		{
			i = *j;
			break;
		}
	}
	auto it = beg;
	for (int i = 0; i < 50; i++)
	{
		it++;
	}

	double ii = (i + *it) / 2;  //!!! Использование явного типа
	while (ii == 0)
	{
		it++;
		ii = (i + *it) / 2;
	}
	for (auto j = beg; j != end; j++)
	{
		*j /= ii;
	}
}


template < typename T >
void modifyForEach(T& cont)
{
	double i; //!!! Использование явного типа
	for (auto j = cont.begin(); j != cont.end(); j++)
	{
		if (*j < 0)
		{
			i = *j;
			break;
		}
	}
	auto it = cont.begin();
	for (int i = 0; i < 50; i++)
	{
		it++;
	}
	double ii = (i + *it) / 2; //!!! Использование явного типа

	std::for_each(cont.begin(), cont.end(), [ii](typename T::value_type& el) { return el /= ii; });
}

template<typename T>
void modifyTransform(const std::string& filename, const std::string& filenameOut)
{
	std::ifstream fin(filename);
	std::vector<typename T::value_type> tempContainer;
	std::copy(std::istream_iterator<typename T::value_type>(fin), std::istream_iterator<typename T::value_type>(), std::back_inserter(tempContainer));
	double i; //!!! Использование явного типа
	for (auto j = tempContainer.begin(); j != tempContainer.end(); j++)
	{
		if (*j < 0)
		{
			i = *j;
			break;
		}
	}
	auto it = tempContainer.begin();
	for (int i = 0; i < 50; i++)
	{
		it++;
	}
	double ii = (i + *it) / 2;
	std::ofstream fout(filenameOut);
	fin.clear();
	fin.seekg(0);

    //!!! Отформатировать 

	std::transform(std::istream_iterator<typename T::value_type>(fin), std::istream_iterator<typename T::value_type>(), std::ostream_iterator<typename T::value_type>(fout, " "),
		[ii](int el) { return el /= ii; });               //!!! Использование явного типа
}

template < typename T >
void outputResultStream(const T& cont, const std::string filename)
{
	std::ofstream fout(filename);
	fout << "-";
	for (auto it = cont.cbegin(); it != cont.cend(); ++it)
	{
		fout << *it << " ";
	}
}

template < typename T >
void outputResultCopy(const T& cont, const std::string& filename)
{
	std::ofstream fout(filename);

	std::copy(cont.begin(), cont.end(), std::ostream_iterator<typename T::value_type>(fout, " "));
}

template < typename T >
void printContainer(const T& container_p)
{
	for (auto it = container_p.cbegin(); it != container_p.cend(); ++it)
		std::cout << *it << " ";
}
